#!/usr/bin/env python

# Interactive histogram resampling tool. Left-click to select bins you want
# to aggregate, then use popup menu to make aggregation. The cyan dotted line
# shows the total number of elements in selected bins. To make a selection the
# click must occur within histogram segments, to unselect click outside the
# histogram

# Copyright 2002: Serge Boiko <serge.boiko@gmx.net>

__version__ = 0.3
__date__ = "Mon Sep 30 16:43:21 2002"

import glob, os, sys

# The next statements assure importation of qwt from the
# local source tree. This is useful for debugging and testing.
globs = glob.glob('../build/lib.*/qwt')
if globs:                               # found PyQwt modules made by distutils
    sys.path.insert(0, os.path.split(globs[0])[0])

from qt import *
from qwt import *
from Numeric import *
from MLab import *
import copy



# some utility functions
def find(a, cond):
    "Matlab's find equivalent"
    els=compress(cond, a, 0)
    ind=nonzero(cond)
    return (ind, els)

def histogram(a, bins):
    "Simple histogram tool"
    n=searchsorted(sort(a), bins)
    n=concatenate([n, [len(a)]])
    
    # sometimes i need average values too
    mean_val = zeros(len(bins), Float)
    for i in range(0, len(bins)-1):
        (ind, el) = find(a, (a>=bins[i])&(a<bins[i+1]))
        try:
            mean_val[i] = mean(el)
        except ZeroDivisionError:
            mean_val[i] = bins[i]

    nums=n[1:]-n[:-1]
    # last elemenst are thrown away!
    return (nums[:-1], mean_val[:-1])



class HistogramWidget(QWidget):
    def __init__(self, data, bins, *args):
        "Data is the data we want to histogram"
        apply(QWidget.__init__, (self,) + args)
        # accumulate selected segments here
        self.selSegms = []
        self.undo = UndoHolder(None, None, None)
        self.data=data
        self.setIcon(QPixmap('hist_icon.xpm'))
        self.setCaption("Interactive Histogram Resampler")
        # make a QwtPlot widget
        self.plot = QwtPlot('Interactive Histogram Resampler', self)
        self.plot.setCanvasBackground(QColor(45, 48, 97))
        # calculate a histogram here
        self.bins = bins
        (self.nums, self.mean_vals) = histogram(self.data, self.bins)
        self.drawHistogram()
        self.plot.setAxisAutoScale(QwtPlot.yLeft)
        self.plot.setAxisAutoScale(QwtPlot.xBottom)
        # set axis titles
        self.plot.setAxisTitle(QwtPlot.xBottom, 'x -->')
        self.plot.setAxisTitle(QwtPlot.yLeft, 'N -->')
        # disable show the legend
        self.plot.enableLegend(0)
        self.pop = HistogramPopup(self)
        self.pop.insertItem("Aggregate Selection", self.aggregateSegms)
       # self.pop.insertItem("Plot with Grace", self.plotGrace)
        self.pop.insertSeparator()
        self.pop.insertItem("Undo!", self.undoSelection)
        self.pop.setItemEnabled(self.pop.idAt(3), 0)
        tt_txt= \
        """<p><b>Left-click</b> to select the bins you want 
        to aggregate. The cyan dotted 
        line shows the total number of elements 
        in the selected bins. To make a selection
        click within the histogram
        segments, to unselect click outside the
        histogram. <br><b>Right-click</b> shows the popup menu
        from which you can choose <b>"Aggregate Selection"</b>
        and <b>"Undo!"</b> actions</p>"""
        
        QToolTip.add(self, tt_txt)
        self.__activateSS()
        self.plot.replot()

    def makeCurveData(self, bins, nums):
        "Calculates points to be plotted as a histogram"
        dX=zeros((len(nums), 5), Float)
        dY=zeros((len(nums), 5), Float)

        for i in range(0, len(nums)):
            dX[i, :] = array([bins[i], bins[i], bins[i + 1], bins[i + 1], bins[i]])
            dY[i, :] = array([0., nums[i], nums[i], 0., 0.])

        return (dX, dY)    

    def whereIam(self, xPos, yPos):
        """If the mouse click occurs inside a histogram segment, returns
        the number of this segment (starting of course from zero), if
        the click occurs outside a histogram returns None"""
        # translate click's coordinate into plots's coordinates
        xPosTr = self.plot.invTransform(QwtPlot.xBottom, xPos)
        yPosTr = self.plot.invTransform(QwtPlot.yLeft, yPos)
        # which elements are to the right
        (ind, els) = find(self.bins, xPosTr <= self.bins)
        
        if (len(ind) == len(self.bins)) or (len(ind) == 0):
            # we missed
            pass
        else:
            pos = min(ind) - 1
            if (yPosTr >= 0) and (yPosTr <= self.nums[pos]):

                # we are inside a segment
                return pos
            else:
                return None
                

    def aggregateSegms(self):
        "Plots aggregated values on the basis of user's selection"
        # throw them away!
        self.undo.update(self.bins, self.data, self.nums)
        put(self.bins, self.selSegms[1:], -111.0)
        self.bins = compress(self.bins >= 0, self.bins)
        # calculate the total number of elements in selected bins
        tnums = sum(self.nums[min(self.selSegms):max(self.selSegms)+1])
        # throw them away!    
        put(self.nums, self.selSegms[1:], -111.0)
        self.nums = compress(self.nums >= 0, self.nums)
        self.selSegms = []    
        self.plot.removeCurves()
        (self.nums, self.mean_vals) = histogram(self.data, self.bins)
        self.drawHistogram()
        self.plot.replot()
        self.pop.setItemEnabled(self.pop.idAt(3), 1)
        
		  
    def drawHistogram(self):
        
        (self.nums, self.mean_vals) = histogram(self.data, self.bins)
        self.plot.removeCurves()
        # prepare data for plotting
        (self.cDataX, self.cDataY)=self.makeCurveData(self.bins, self.nums)
        
        # insert curves:
        self.curb=[]
        for i in range(0, len(self.nums)):
            self.curb.append(self.plot.insertCurve(str(i)))
            self.plot.setCurvePen(self.curb[i], QPen(Qt.yellow, 2))
            self.plot.setCurveData(self.curb[i], self.cDataX[i, :], self.cDataY[i, :])
            
            # draw a level line to show the num of elms in selection
            self.levelLine = self.plot.insertCurve("LevelLine")
            self.plot.setCurvePen(self.levelLine, QPen(Qt.yellow, 2))
        
    def plotMousyPressed(self, e):
        "Mouse press processing instructions go here"
        if e.button() == QMouseEvent.LeftButton:
            self.xPos = e.pos().x()
            self.yPos = e.pos().y()
        
            pos = self.whereIam(self.xPos, self.yPos)
        
            if pos is not None:
                self.selSegms.append(pos)
                sortSelSegms = sort(self.selSegms)
                for i in range(min(sortSelSegms), max(sortSelSegms) + 1, 1):
                    self.plot.setCurvePen(self.curb[i], QPen(Qt.red, 2))
                    self.plot.replot()
                self.selSegms = range(min(sortSelSegms), max(sortSelSegms) + 1, 1)
                # total number of elms in selection
                tnums = sum(self.nums[min(self.selSegms):max(self.selSegms) + 1])
                self.plot.setCurvePen(self.levelLine, QPen(Qt.darkCyan, 1, Qt.DotLine))
                # get axis bounds
                scD = self.plot.axisScale(QwtPlot.xBottom)
                self.plot.setCurveData(
                    self.levelLine, [scD.lBound(), scD.hBound()], [tnums, tnums])
                self.plot.replot()
            else:
                for i in self.selSegms:
                    self.plot.setCurvePen(self.curb[i], QPen(Qt.yellow, 2))
                    self.selSegms = []
                    self.plot.setCurveData(
                    self.levelLine, [min(self.bins), max(self.bins)],
                    [0., 0.])
                    # "hide" the level line
                    self.plot.setCurvePen(self.levelLine, QPen(Qt.yellow, 2))
                    self.plot.replot()
        else:
            #do nothing:
            pass

    def plotMousyReleased(self, e):
        self.aggregateSegms()

    def __activateSS(self):
       QObject.connect(self.plot, SIGNAL('plotMousePressed(const QMouseEvent&)'),
                        self.plotMousyPressed) 
    
       QObject.connect(self.plot, SIGNAL('plotMousePressed(const QMouseEvent&)'),
                       self.pop.raisePop)
        
       #QObject.connect(self.plot, SIGNAL('plotMouseReleased(const QMouseEvent&)'),
       #               self.plotMousyReleased)
    def undoSelection(self):
        "Undo last changes"
        if self.undo.prevBins is not None:
            self.bins = self.undo.prevBins
        self.nums = self.undo.prevNums
        self.drawHistogram()
        
        #self.undo.clear()
        self.selSegms = []
        self.plot.setCurveData(
            self.levelLine, [min(self.bins), max(self.bins)],
            [0., 0.])
        # "hide" the level line
        self.plot.setCurvePen(self.levelLine, QPen(Qt.yellow, 2))
        #self.undo.clear()
        self.plot.replot()

  

    def resizeEvent(self, e):
        self.plot.resize(e.size())
        self.plot.move(0,0)


class HistogramPopup(QPopupMenu):
    def __init__(self, *args):
        apply(QPopupMenu.__init__, (self, ) + args)

    def raisePop(self, evt):
        """Raise popup only if right mouse pressed"""
        if evt.button()==QMouseEvent.RightButton:
            # we need to use global position here!
            self.popup(evt.globalPos())


class UndoHolder:
	def __init__(self, bins, data, nums):
		"Constructs a holder to keep previous state"
		self.prevBins = copy.copy(bins)
		self.prevNums = copy.copy(nums)
		self.prevData = copy.copy(data)
	def update(self, newBins, newData, newNums):
		"Update the state of holder"
		self.prevBins = copy.copy(newBins)
		self.prevNums = copy.copy(newNums)
		self.prevData = copy.copy(newData)
	def clear(self):
		"Remove contens"
		self.prevBins = None
		self.prevData = None



# admire:
if __name__ == "__main__":
    app = QApplication(sys.argv)
    data = rand(300)*10.
    bins = arange(0, 10.5, .5)
    demo = HistogramWidget(data, bins)
    app.setMainWidget(demo)
    demo.show()
    app.exec_loop()

