#!/usr/bin/env python

import glob, os, sys

# The next statements assure importation of qwt from the
# local source tree. This is useful for debugging and testing.
globs = glob.glob('../build/lib.*/qwt')
if globs:                               # found PyQwt modules made by distutils
    sys.path.insert(0, os.path.split(globs[0])[0])

from qt import *
from qwt import *
from types import *

__all__ = [
    # classes
    'Curve',
    'Pen',
    'Plot',
    'Symbol',
    # colors
    'black',
    'blue',
    'cyan',
    'darkBlue',
    'darkCyan',
    'darkGray',
    'darkGreen',
    'darkMagenta',
    'darkRed',
    'darkYellow',
    'gray',
    'green',
    'lightGray',
    'magenta',
    'red',
    'white',
    'yellow',
    # axis indicators
    'left',
    'right',
    'bottom',
    'top',
    # symbols
    'circles',
    'squares'
    ]

# colors
black       = Qt.black
blue        = Qt.blue
cyan        = Qt.cyan
darkBlue    = Qt.darkBlue
darkCyan    = Qt.darkCyan
darkGray    = Qt.darkGray
darkGreen   = Qt.darkGreen
darkMagenta = Qt.darkMagenta
darkRed     = Qt.darkRed
darkYellow  = Qt.darkYellow
gray        = Qt.gray
green       = Qt.green
lightGray   = Qt.lightGray
magenta     = Qt.magenta
red         = Qt.red
white       = Qt.white
yellow      = Qt.yellow

# axis indicator
left        = QwtPlot.yLeft
right       = QwtPlot.yRight 
bottom      = QwtPlot.xBottom
top         = QwtPlot.xTop

# Helper
def isSequenceOfFloats(x):
    pass

# Curve is based on scipy/xplt/curve

class Curve:
    """
    Curve offers a simplified interface to QwtPlotCurve.
    """
    def __init__(self, x, y, *arguments, **keywords):
        """
        curve = Curve(x, y, *arguments, **keywords)
        obligatory arguments:
        -- x must be a sequence of floats
        -- y must be a sequence of floats
        optional arguments in any order, dispatch on argument type:
        -- name is a Python string or a QString
        -- color is a QColor
        optional keyword argument in any order, dispatch on keyword:
        -- xaxis is xBottom (default) or xTop
        -- yaxis is yLeft (default) of yRight
        """
        self.x = x # must be sequence of floats, typecode()?
        self.y = y # must be sequence of floats
        self.name = ""
        self.color = black
        self.xaxis = bottom
        self.yaxis = left
        self.symbol = None
        self.pen = None

        for argument in arguments:
            if isinstance(argument, Symbol):
                self.symbol = argument()
            elif isinstance(argument, QColor):
                self.color = argument
            elif isinstance(argument, Pen):
                self.pen = argument()
            elif (isinstance(argument, StringType)
                  or isinstance(argument, QString)):
                self.name = argument

        if keywords.has_key("xaxis") and keywords["xaxis"] == top:
            self.xaxis = top
        if keywords.has_key("yaxis") and keywords["yaxis"] == right:
            self.yaxis = right

        if not self.symbol and not self.pen:
            self.pen = QPen(self.color)
        if self.symbol:
            self.symbol.setBrush(QBrush(self.color))

class Symbol:
    def __init__(self, *arguments, **keywords):
        self.keywords = keywords

    def __call__(self):
        symbol = QwtSymbol()
        if self.keywords.has_key('pen'):
            symbol.setPen(self.keywords['pen'])
        if self.keywords.has_key('style'):
            symbol.setStyle(self.keywords['style'])
        if self.keywords.has_key('size'):
            symbol.setSize(self.keywords['size'])
        else:
            symbol.setSize(5)
        return symbol
            
circles = Symbol(style = QwtSymbol.Ellipse)
squares = Symbol(style = QwtSymbol.Rect)

class Pen:
    def __init__(self, *arguments, **keywords):
        self.arguments = arguments
        self.keywords = keywords

    def __call__(self):
        pen = QPen()
        for argument in self.arguments:
            if isinstance(argument, QColor):
                pen.setColor(argument)
        if self.keywords.has_key('style'):
            pen.setStyle(self.keywords['style'])
        if self.keywords.has_key('width'):
            pen.setWidth(self.keywords['width'])
        return pen

class Plot(QwtPlot):
    """
    Plot offers a simplified user interface to QwtPlot.
    """
    def __init__(self, *curves, **keywords):
        """
        p = Plot(*curves, **keywords)
        optional arguments:
        -- any number of curves
        optional keyword arguments in any order, dispatch on keyword
        -- size
        -- title
        """
        QwtPlot.__init__(self)
        # user interface
        self.setCanvasBackground(Qt.white)
        self.setOutlinePen(QPen(Qt.green))
        self.setAutoLegend(1)
        self.setLegendPos(Qwt.Right)

        self.size = (600, 400)
        self.title = ""
        
        self.__plot(curves)
        self.__set(keywords)

        apply(self.resize, self.size)
        self.setTitle(self.title)
        self.replot()

        self.connect(self,
                     SIGNAL('plotMouseMoved(const QMouseEvent&)'),
                     self.onMouseMoved)
        self.show()

    def plot(self, *curves):
        self.__plot(curves)
        self.replot()
        
    def __plot(self, curves):
        for curve in curves:
            if curve.xaxis == top:
                self.enableAxis(top)
            if curve.yaxis == right:
                self.enableAxis(right)
            key = self.insertCurve(curve.name, curve.xaxis, curve.yaxis)
            if curve.pen:
                self.setCurvePen(key, curve.pen)
            else:
                self.setCurveStyle(key, QwtCurve.NoCurve)
            if curve.symbol:
                self.setCurveSymbol(key, curve.symbol)
            self.setCurveData(key, curve.x, curve.y)

    def set(self, **keywords):
        self.__set(keywords)
        apply(self.resize, self.size)
        self.setTitle(self.title)
        self.replot()
        
    def __set(self, keywords):
        if keywords.has_key('size'):
            self.size = keywords["size"]
        if keywords.has_key('title'):
            self.title = keywords["title"]

    def setAxis(self, axis, *arguments, **keywords):
        self.__setAxis(axis, arguments, keywords)
        self.replot()

    def __setAxis(self, axis, arguments, keywords):
        if keywords.has_key('log'):
            self.changeAxisOptions(
                axis, QwtAutoScale.Logarithmic, keywords['log'])
        
    def erase(self, key):
        pass

    def onMouseMoved(self, e):
        x = self.invTransform(bottom, 2+e.pos().x())
        y = self.invTransform(left, 2+e.pos().y())
        print 'x=%g, y=%g' % (x, y)

        
# Admire!
def test():
    x = arange(-2*pi, 2*pi, 0.01)
    p = Plot(Curve(x, cos(x), blue, "cos(x)"),
             Curve(x, exp(x), green, "exp(x)", yaxis=right),
             title="Demo")
    p.setAxis(right, log=1)
    x = arange(-2*pi, 2*pi, 0.1)
    pen = Pen(magenta, width=3, style=Qt.SolidLine)
    p.plot(Curve(x, cos(x-pi/4), yellow, circles, "circles"),
           Curve(x, cos(x+pi/4), pen, cyan, squares, "squares"))
    return p

if __name__ == '__main__':
    from Numeric import *
    try: 
        qApp.argc() 
        p = test()
    except RuntimeError:
        a = QApplication(sys.argv)
        p = test()
        a.setMainWidget(p)
        a.exec_loop()

    
