#!/usr/bin/env python

"""This module provides an interpreter friendly interface to QwtPlot.

Plot

Curve

Axis

Pen

Symbol

Brush
"""

import glob, os, sys

# The next statements assure importation of qwt from the
# local source tree. This is useful for debugging and testing.
globs = glob.glob('../build/lib.*/qwt')
if globs:                               # found PyQwt modules made by distutils
    sys.path.insert(0, os.path.split(globs[0])[0])

from Numeric import *
from qt import *
from qwt import *
from types import *

"""
__all__ = [
    # classes
    'Plot',
    'Curve',
    'Symbol',
    'Axis',
    'Brush',
    'Pen',
    # colors
    'Black',
    'Blue',
    'Cyan',
    'DarkBlue',
    'DarkCyan',
    'DarkGray',
    'DarkGreen',
    'DarkMagenta',
    'DarkRed',
    'DarkYellow',
    'Gray',
    'Green',
    'LightGray',
    'Magenta',
    'Red',
    'White',
    'Yellow',
    # axis orientation
    'Left',
    'Right',
    'Bottom',
    'Top',
    # axis option
    'PlainAxis',
    'IncludeRef',
    'Symmetric',
    'Floating',
    'Logarithmic',
    'Inverted',
    'Logarithmic',
    # symbol style
    'NoSymbol',
    'Circle',
    'Square',
    'Diamond',
    # line style
    'NoLine',
    'SolidLine',
    'DashLine',
    'DotLine',
    'DashDotLine',
    'DashDotDotLine',
    ]
"""
# colors
Black       = Qt.black
Blue        = Qt.blue
Cyan        = Qt.cyan
DarkBlue    = Qt.darkBlue
DarkCyan    = Qt.darkCyan
DarkGray    = Qt.darkGray
DarkGreen   = Qt.darkGreen
DarkMagenta = Qt.darkMagenta
DarkRed     = Qt.darkRed
DarkYellow  = Qt.darkYellow
Gray        = Qt.gray
Green       = Qt.green
LightGray   = Qt.lightGray
Magenta     = Qt.magenta
Red         = Qt.red
White       = Qt.white
Yellow      = Qt.yellow



class PositonMarker(QwtPlotMarker):
    def __init__(self, *args):
        apply(QwtPlotMarker.__init__, (self,) + args)

    def draw(self, painter, x, y, rect):
        pass
        
    
class Plot(QwtPlot):
    """Sugar coating for QwtPlot.
    
    """
    def __init__(self, *args):
        """Constructor.

        Plot takes in any order any number of
        (1) Curves              --> data points,
        (2) strings or QStrings --> title,
        (3) tuples of 2 ints    --> size.
        """
        self.size = (600, 400)
        # get an optional parent widget
        parent = None
        for arg in args:
            if isinstance(arg, QWidget):
                parent = arg
                self.size = None
        QwtPlot.__init__(self, parent)

        #self.tip = QToolTip(self)
        self.marker = self.insertMarker()
        self.setMarkerLineStyle(self.marker, QwtMarker.Cross)
        self.setMarkerLabelPen(self.marker, QPen(Qt.red, 3))
        
        # user interface
        self.setCanvasBackground(Qt.white)
        self.setOutlinePen(QPen(Qt.green))
        self.setAutoLegend(1)
        self.setLegendPos(Qwt.Right)
        
        for arg in args:
            if isinstance(arg, Curve):
                self.__plot(arg)
            elif (isinstance(arg, StringType) or isinstance(arg, QString)):
                self.setTitle(arg)
            elif (isinstance(arg, tuple) and len(tuple) == 2
                  and isinstance(arg[0], int) and isinstance(arg[1], int)):
                self.size = arg
            elif (isinstance(arg, QWidget)):
                pass
            else:
                print "Plot() fails to accept" % arg

        if self.size:
            apply(self.resize, self.size)
        self.replot()

        self.connect(self, SIGNAL('plotMousePressed(const QMouseEvent&)'),
                     self.onMousePressed)
        self.connect(self, SIGNAL("legendClicked(long)"), self.toggleCurve)
        self.show()

    def plot(self, *args):
        for arg in args:
            if isinstance(arg, Curve):
                self.__plot(arg)
            else:
                print "Plot.plot() fails to accept", arg
        self.replot()
        
    def __plot(self, curve):
        xAxis, xOptions = curve.xAxis
        self.enableAxis(xAxis)
        self.setAxisOptions(xAxis, xOptions)
        yAxis, yOptions = curve.yAxis
        self.enableAxis(yAxis)
        self.setAxisOptions(yAxis, yOptions)
        key = self.insertCurve(curve.name, xAxis, yAxis)
        if curve.pen:
            self.setCurvePen(key, curve.pen)
        else:
            self.setCurveStyle(key, QwtCurve.NoCurve)
        if curve.symbol:
            self.setCurveSymbol(key, curve.symbol)
        self.setCurveData(key, curve.x, curve.y)

    def onMousePressed(self, e):
        x, y = e.pos().x(), e.pos().y()
        done, xs, ys = [], [], []
        todo = ((QwtPlot.xBottom, "x0=%+.4g", x, xs),
                (QwtPlot.yLeft,   "y0=%+.4g", y, ys),
                (QwtPlot.xTop,    "x1=%+.4g", x, xs),
                (QwtPlot.yRight,  "y1=%+.4g", y, ys))
        for axis, template, value, fifo in todo:
            if self.axisEnabled(axis):
                value = self.invTransform(axis, value)
                done.append(template % value)
                fifo.append(value)
        if x < self.canvas().width()/2:
            align = Qt.AlignRight
        else:
            align = Qt.AlignLeft
        if y < self.canvas().height()/2:
            align |= Qt.AlignBottom
        else:
            align |= Qt.AlignTop
        self.setMarkerLabelAlign(self.marker, align)
        self.setMarkerPos(self.marker, xs[0], ys[0])
        self.setMarkerLabel(self.marker, '; '.join(done))
        self.replot()
        #self.tip.tip(self.rect(), '\n'.join(done))

    def toggleCurve(self, key):
        curve = self.curve(key)
        if curve:
            curve.setEnabled(not curve.enabled())
            self.replot()
        
class Curve:
    """Sugar coating for QwtPlotCurve.
    
    """
    def __init__(self, x, y, *args):
        """
        curve = Curve(x, y, *arguments)
        obligatory arguments:
        -- x must be a sequence of floats
        -- y must be a sequence of floats
        optional arguments in any order, dispatch on argument type:
        (1) Axis
        (2) Pen
        (3) Symbol
        (4) string or QString
        """
        self.x = x # must be sequence of floats, typecode()?
        self.y = y # must be sequence of floats
        self.name = ""
        self.xAxis = (QwtPlot.xBottom, QwtAutoScale.None)
        self.yAxis = (QwtPlot.yLeft, QwtAutoScale.None)
        self.symbol = None
        self.pen = None

        for arg in args:
            if isinstance(arg, Axis):
                if arg.orientation in (QwtPlot.xBottom, QwtPlot.xTop):
                    self.xAxis = (arg.orientation, arg.options)
                elif arg.orientation in (QwtPlot.yLeft, QwtPlot.yRight):
                    self.yAxis = (arg.orientation, arg.options)
            elif isinstance(arg, Symbol):
                self.symbol = arg
            elif isinstance(arg, Pen):
                self.pen = arg
            elif (isinstance(arg, StringType) or isinstance(arg, QString)):
                self.name = arg
            else:
                print "Curve fails to accept %s" % arg

        if not self.symbol and not self.pen:
            self.pen = QPen()

class AxisOption:
    def __init__(self, option):
        self.option = option
    # fixme 

PlainAxis    = QwtAutoScale.None
IncludeRef   = QwtAutoScale.IncludeRef
Symmetric    = QwtAutoScale.Symmetric
Floating     = QwtAutoScale.Floating
Logarithmic  = QwtAutoScale.Logarithmic
Inverted     = QwtAutoScale.Inverted

class AxisOrientation:
    def __init__(self, orientation):
        self.orientation = orientation

Left   = AxisOrientation(QwtPlot.yLeft)
Right  = AxisOrientation(QwtPlot.yRight)
Bottom = AxisOrientation(QwtPlot.xBottom)
Top    = AxisOrientation(QwtPlot.xTop)

class Axis:
    def __init__(self, *args):
        for arg in args:
            if isinstance(arg, AxisOrientation):
                self.orientation = arg.orientation
            elif isinstance(arg, int):
                self.options = arg
            else:
                print "Symbol() fails to accept" %  arg


class Brush:
    pass

class SymbolStyle:
    def __init__(self, style):
        self.style = style

NoSymbol = SymbolStyle(QwtSymbol.None)
Circle   = SymbolStyle(QwtSymbol.Ellipse)
Square   = SymbolStyle(QwtSymbol.Rect)
Diamond  = SymbolStyle(QwtSymbol.Diamond)

class PenStyle:
    def __init__(self, style):
        self.style = style

NoLine         = PenStyle(Qt.NoPen) 
SolidLine      = PenStyle(Qt.SolidLine)
DashLine       = PenStyle(Qt.DashLine)
DotLine        = PenStyle(Qt.DotLine)
DashDotLine    = PenStyle(Qt.DashDotLine)
DashDotDotLine = PenStyle(Qt.DashDotDotLine)
    
class Symbol(QwtSymbol):
    def __init__(self, *args):
        QwtSymbol.__init__(self)
        self.setSize(5)
        for arg in args:
            if isinstance(arg, QColor):
                brush = self.brush()
                brush.setColor(arg)
                self.setBrush(brush)
            elif isinstance(arg, SymbolStyle):
                self.setStyle(arg.style)
            elif isinstance(arg, int):
                self.setSize(arg)
            else:
                print "Symbol fails to accept %s" %  arg


class Pen(QPen):
    def __init__(self, *args):
        QPen.__init__(self)
        for arg in args:
            if isinstance(arg, QColor):
                self.setColor(arg)
            elif isinstance(arg, int):
                self.setWidth(arg)
            else:
                print "Pen fails to accept %s" % arg


print_xpm = ['32 32 12 1',
             'a c #ffffff',
             'h c #ffff00',
             'c c #ffffff',
             'f c #dcdcdc',
             'b c #c0c0c0',
             'j c #a0a0a4',
             'e c #808080',
             'g c #808000',
             'd c #585858',
             'i c #00ff00',
             '# c #000000',
             '. c None',
             '................................',
             '................................',
             '...........###..................',
             '..........#abb###...............',
             '.........#aabbbbb###............',
             '.........#ddaaabbbbb###.........',
             '........#ddddddaaabbbbb###......',
             '.......#deffddddddaaabbbbb###...',
             '......#deaaabbbddddddaaabbbbb###',
             '.....#deaaaaaaabbbddddddaaabbbb#',
             '....#deaaabbbaaaa#ddedddfggaaad#',
             '...#deaaaaaaaaaa#ddeeeeafgggfdd#',
             '..#deaaabbbaaaa#ddeeeeabbbbgfdd#',
             '.#deeefaaaaaaa#ddeeeeabbhhbbadd#',
             '#aabbbeeefaaa#ddeeeeabbbbbbaddd#',
             '#bbaaabbbeee#ddeeeeabbiibbadddd#',
             '#bbbbbaaabbbeeeeeeabbbbbbaddddd#',
             '#bjbbbbbbaaabbbbeabbbbbbadddddd#',
             '#bjjjjbbbbbbaaaeabbbbbbaddddddd#',
             '#bjaaajjjbbbbbbaaabbbbadddddddd#',
             '#bbbbbaaajjjbbbbbbaaaaddddddddd#',
             '#bjbbbbbbaaajjjbbbbbbddddddddd#.',
             '#bjjjjbbbbbbaaajjjbbbdddddddd#..',
             '#bjaaajjjbbbbbbjaajjbddddddd#...',
             '#bbbbbaaajjjbbbjbbaabdddddd#....',
             '###bbbbbbaaajjjjbbbbbddddd#.....',
             '...###bbbbbbaaajbbbbbdddd#......',
             '......###bbbbbbjbbbbbddd#.......',
             '.........###bbbbbbbbbdd#........',
             '............###bbbbbbd#.........',
             '...............###bbb#..........',
             '..................###...........']

class Fancy(QMainWindow):
    def __init__(self, *args):
        QMainWindow.__init__(self)
        self.qplt = apply(Plot, (self,)+args)
        self.setCentralWidget(self.qplt)

        self.toolBar = QToolBar(self)

        printButton = QToolButton(self.toolBar)
        printButton.setTextLabel("Print")
        #printButton.setPixmap(QPixmap(print_xpm))
        printButton.setUsesTextLabel(1)
        self.resize(700, 500)
        self.connect(printButton, SIGNAL('clicked()'), self.printPlot)
        self.show()

    def plot(self, *args):
        self.qplt.plot(*args)

    def axisScale(self, orientation):
        return self.qplt.axisScale(orientation)
    
    def printPlot(self):
        p = QPrinter()
        if p.setup():
            self.qplt.printPlot(p)
            
# Admire!
def test():
    x = arange(-2*pi, 2*pi, 0.01)
    p = Fancy(Curve(x, cos(x), Pen(Magenta, 2), "cos(x)"),
              Curve(x, exp(x), Pen(Red), "exp(x)", Axis(Right, Logarithmic)),
              "PyQwt demo based on Qwt-%s" % QWT_VERSION_STR)
    x = x[0:-1:10]
    p.plot(Curve(x, cos(x-pi/4), Symbol(Circle, Yellow), "circle"),
           Curve(x, cos(x+pi/4), Pen(Blue), Symbol(Square, Cyan), "square"))
    return p

if __name__ == '__main__':
    # HACK to allow execfile('qplt.py') from a PyQt application (PyCute.py)
    try: 
        qApp.argc() 
        p = test()
    except RuntimeError:
        a = QApplication(sys.argv)
        p = test()
        a.setMainWidget(p)
        a.exec_loop()

    
