#!/usr/bin/env python

import glob, os, sys

# The next statements assure importation of qwt from the
# local source tree. This is useful for debugging and testing.
globs = glob.glob('../build/lib.*/qwt')
if globs:                               # found PyQwt modules made by distutils
    sys.path.insert(0, os.path.split(globs[0])[0])

from qt import *
from qwt import *
from Numeric import *
#from xLinearAlgebra import *
from LinearAlgebra import *

# pipo
ch = 0.2
cv = 1.0
lv = 1.0
lh = 0.4
lq = 0.002

# realistic
ch = 1.25e-12
cv = 21.175e-12
lv = 10.000e-9
lh =  3.000e-9

#_ground
#_input

class element:
    def __init__(self, n1, n2, value):
        self.lo = min(n1, n2)
        self.hi = max(n1, n2)
        self.value = value

    def hiNode(self):
        return self.hi

    def loNode(self):
        return self.lo
    

class capacitor(element):
    def __init__(self, n1, n2, value):
        element.__init__(self, n1, n2, value)
        
    def conductanceLoHi(self, omega):
        return 1j*omega*self.value, self.lo, self.hi


class inductor(element):
    def __init__(self, n1, n2, value):
        element.__init__(self, n1, n2, value)

    def conductanceLoHi(self, omega):
        return 1.0/((lq+1j)*omega*self.value), self.lo, self.hi


class resistor(element):
    def __init__(self, n1, n2, value):
        element.__init__(self, n1, n2, value)

    def conductanceLoHi(self, omega):
        return 1.0/self.value, self.lo, self.hi


def rhs(omega):
    return 2*sqrt((1-omega*omega*cv*lv)/(lh*cv))

# input node = -1
# ground node = -2

def zMatrix(circuit, omega):
    # find the number of modes and the node connected to the "input" branch
    #highs = apply(hiNode, circuit)
    highs = [0]
    for element in circuit:
        highs.append(element.hiNode())
        if element.loNode() == -1:
            inNode = element.hiNode()
    nodes = 1 + max(highs)
    # the hard work
    z = zeros(len(omega), Complex)
    for i in range(len(omega)):
        a = zeros((nodes, nodes), Complex)  # nodal conductance matrix
        b = zeros(nodes, Complex)           # source current vector
        for element in circuit:
            g, lo, hi = element.conductanceLoHi(omega[i])
            if lo == -2:                    # ground
                a[hi][hi] += g
            elif lo == -1:                  # input
                b[hi] += g
                a[hi][hi] += g
            else:
                a[lo][lo] += g
                a[hi][hi] += g
                a[lo][hi] -= g
                a[hi][lo] -= g
        v = solve_linear_equations(a, b)
        z[i] = 1.0/(b[inNode]*(1-v[inNode]))
    return z


class SmithChart(QWidget):
    def __init__(self,*args):
        apply(QWidget.__init__,(self,) + args)
	
	# The QwtPlot widget
	self.griPlot = QwtPlot("Smith Chart", self)
        self.gnfPlot = QwtPlot("abs(Gamma) - f", self)
        self.zPlot = QwtPlot("re(Z), im(Z) - f", self)
        self.rPlot = QwtPlot("Intersect Plot", self)
        # FIXME : look in .sip for the last option (step): maybe double = 0.0
        self.griPlot.setAxisScale(QwtPlot.xBottom, -1.0, 1.0)
        self.griPlot.setAxisScale(QwtPlot.yLeft, -1.0, 1.0)
	# self.zPlot.setAxisScale(QwtPlot.xBottom, 300, 400)
        
        # circuit description
        circuit = [
            capacitor( -1,  1, ch),
            capacitor(  0,  1, cv),
            inductor ( -2,  1, lv),
            inductor (  0,  2, lh),
            capacitor(  2,  3, cv),
            inductor ( -2,  3, lv),
            inductor (  2,  4, lh),
            capacitor(  4,  5, cv),
            inductor ( -2,  5, lv),
            inductor (  4,  6, lh),
            capacitor(  6,  7, cv),
            inductor ( -2,  7, lv),
            inductor (  6,  8, lh),
            capacitor(  8,  9, cv),
            inductor ( -2,  9, lv),
            inductor (  8, 10, lh),
            capacitor( 10, 11, cv),
            inductor ( -2, 11, lv),
            inductor ( 10,  0, lh),
            ]

        fmax = 1.0/(2*pi*sqrt(lv*cv))
        fs = 2*pi*arrayrange(3*fmax/4, fmax, 50e3)
        zs = zMatrix(circuit, fs)
        gs = (zs-50.0)/(zs+50.0)
        sect = rhs(fs)/(2e6*pi)
        fs = fs/(2e6*pi)

        curveSnail = self.griPlot.insertCurve("im(Gamma) vs re(Gamma)")
        self.griPlot.setCurveData(curveSnail, gs.real, gs.imag)
        curveAbsG = self.gnfPlot.insertCurve("abs(Gamma) vs f")
        self.gnfPlot.setCurveData(curveAbsG, fs, abs(gs))
        curveReLadder = self.zPlot.insertCurve("re(zLadder)")
        curveImLadder = self.zPlot.insertCurve("im(zLadder)")
        self.zPlot.setCurveData(curveReLadder, fs, zs.real)
        self.zPlot.setCurvePen(curveReLadder, QPen(Qt.red))
        self.zPlot.setCurveData(curveImLadder, fs, zs.imag)
        self.zPlot.setCurvePen(curveImLadder, QPen(Qt.blue))
        curveSect1 = self.rPlot.insertCurve("mode 1")
        curveSect2 = self.rPlot.insertCurve("mode 2")
        curveSect3 = self.rPlot.insertCurve("mode 3")
        curveSectR = self.rPlot.insertCurve("intersect")
        self.rPlot.setCurveData(curveSectR, fs, fs)
        self.rPlot.setCurvePen(curveSectR, QPen(Qt.red))
        self.rPlot.setCurveData(curveSect1, fs, sin(pi/6)*sect)
        self.rPlot.setCurvePen(curveSect1, QPen(Qt.green))
        self.rPlot.setCurveData(curveSect2, fs, sin(pi/3)*sect)
        self.rPlot.setCurvePen(curveSect2, QPen(Qt.green))
        self.rPlot.setCurveData(curveSect3, fs, sin(pi/2)*sect)
        self.rPlot.setCurvePen(curveSect3, QPen(Qt.green))

    def resizeEvent(self, e):
        x = e.size().width()/2
        y = e.size().height()/2
    	self.griPlot.resize(x, y)
	self.griPlot.move(0, 0)
    	self.gnfPlot.resize(x, y)
	self.gnfPlot.move(0, y)
        self.zPlot.resize(x, y)
        self.zPlot.move(x, 0)
        self.rPlot.resize(x, y)
        self.rPlot.move(x, y)
	

if __name__ == '__main__':
    app = QApplication(sys.argv)
    chart = SmithChart()
    app.setMainWidget(chart)
    chart.resize(700, 700)
    chart.show()
    app.exec_loop()
