/*-*- c++ -*-******************************************************************
 * Qwt Widget Library
 * Copyright (C) 1997   Josef Wilgen
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *****************************************************************************/

#include <qlabel.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include "qwt_painter.h"
#include "qwt_legend.h"
#include "qwt_plot.h"
#include "qwt_plot_layout.h"
#include "qwt_plot_dict.h"
#include "qwt_rect.h"
#include "qwt_scale.h"
#include "qwt_math.h"

/*!
  \brief Print the plot to a \c QPaintDevice (\c QPrinter)
  This function prints the contents of a QwtPlot instance to
  \c QPaintDevice object. The size is derived from its device
  metrics.

  \param paintDev device to paint on, often a printer
  \param flags flags must be a bitwise OR of \c Qwt::PrintFlags values
  \param pfilter print filter
  \sa QwtPlot::print
  \sa QwtPlotPrintFilter
*/

void QwtPlot::print(QPaintDevice &paintDev,
    const QwtPlotPrintFilter &pfilter) const
{
    QPaintDeviceMetrics mpr(&paintDev);

    QRect rect(0, 0, mpr.width() - 1, mpr.height() - 1);
    rect.setHeight(qwtMin(rect.height(), int(0.66 * double(rect.width()))));

    QPainter p(&paintDev);
    print(&p, rect, pfilter);
}
/*!
  \brief Paint the plot into a given rectangle.
  Paint the contents of a QwtPlot instance into a given rectangle.

  \param painter Painter
  \param plotRect Bounding rectangle
  \param flags flags must be a bitwise OR of \c Qwt::PrintFlags values
  \param pfilter Print filter
  \sa QwtPlotPrintFilter
*/
void QwtPlot::print(QPainter *painter, const QRect &plotRect,
        const QwtPlotPrintFilter &pfilter) const
{
	int axis;

    if ( painter == 0 || !painter->isActive() ||
            !plotRect.isValid() || size().isNull() )
       return;

    const QwtScale *axes[axisCnt];
    for ( axis = 0; axis < axisCnt; axis++ )
        axes[axis] = d_axisEnabled[axis] ? d_scale[axis] : 0;
 
    const int options = QwtPlotLayout::IgnoreScrollbars 
        | QwtPlotLayout::IgnoreFrames | QwtPlotLayout::AlignScales;

    painter->save();
    QwtPainter::setScaleMetrics(this, painter->device());

    d_layout->activate(QwtPainter::invScale(plotRect), pfilter, d_lblTitle, 
#ifndef QWT_NO_LEGEND
        d_legend, 
#endif
        axes, d_canvas, options);

    if ((pfilter.options() & QwtPlotPrintFilter::PrintTitle)
        && (!d_lblTitle->text().isEmpty()))
    {
        painter->setFont(pfilter.font(d_lblTitle->font(),
            QwtPlotPrintFilter::Title));
        QwtPainter::drawText(painter, d_layout->titleRect(), 
            d_lblTitle->alignment(), d_lblTitle->text());
    }

#ifndef QWT_NO_LEGEND
    if ( (pfilter.options() & QwtPlotPrintFilter::PrintLegend)
        && d_legend->itemCnt() > 0 )
    {
        printLegend(painter, d_layout->legendRect(), 
            d_layout->canvasRect(), pfilter);
    }
#endif

    for ( axis = 0; axis < axisCnt; axis++ )
    {
        if (d_scale[axis])
        {
            int startDist, endDist;
            d_scale[axis]->minBorderDist(startDist, endDist);

            int baseDist = d_scale[axis]->baseLineDist();
            if ( !(pfilter.options() & QwtPlotPrintFilter::PrintBackground) )
                baseDist = 0;

            printScale(painter, axis, startDist, endDist,
                baseDist, d_layout->scaleRect(axis), pfilter);
        }
    }

    //
    // build maps
    //

    QwtDiMap map[axisCnt];
    for (int i=0; i<axisCnt; i++)
    {
        map[i].setDblRange(d_sdiv[i].lBound(),
            d_sdiv[i].hBound(), d_sdiv[i].logScale());

        const int sDist = d_scale[i]->startBorderDist();
        const int eDist = d_scale[i]->endBorderDist();

        const QRect &scaleRect = d_layout->scaleRect(i);
        if ( i == xTop || i == xBottom )
        {
            map[i].setIntRange(
                int(floor(0.5 + (scaleRect.left() + sDist) * QwtPainter::scaleMetricsX())),
                int(floor(0.5 + (scaleRect.right() - eDist) * QwtPainter::scaleMetricsX()))
            );
        }
        else
        {
            map[i].setIntRange(
                int(floor(0.5 + (scaleRect.bottom() - sDist) * QwtPainter::scaleMetricsY())),
                int(floor(0.5 + (scaleRect.top() + eDist) * QwtPainter::scaleMetricsY()))
            );
        }
    }

	const QRect canvasRect = QwtPainter::scale(d_layout->canvasRect());
    QwtPainter::resetScaleMetrics();

    if ( pfilter.options() & QwtPlotPrintFilter::PrintBackground )
    {
        QColor c = pfilter.color(canvasBackground(),
            QwtPlotPrintFilter::Background);
        QwtPainter::fillRect(painter, canvasRect, c);
    }
    else
        QwtPainter::drawRect(painter, canvasRect);


    printCanvas(painter, map, canvasRect, pfilter);

    painter->restore();

    d_layout->invalidate();
}

/*!
  \brief Paint the legend into a given rectangle.
  Paint the legend into a given rectangle.

  \param painter Painter
  \param rect Bounding rectangle
  \param pfilter Print filter
  \sa QwtPlotPrintFilter
*/

void QwtPlot::printLegend(
#ifndef QWT_NO_LEGEND
QPainter *painter, const QRect &rect,
    const QRect &pixRect, const QwtPlotPrintFilter &pfilter
#else
QPainter *, const QRect &, const QRect &, const QwtPlotPrintFilter &
#endif
) const
{
#ifndef QWT_NO_LEGEND
    if ( d_legend->itemCnt() == 0 )
        return;

    const QFont f = pfilter.font(d_legend->font(), QwtPlotPrintFilter::Legend);

    const QSize cellSize = d_legend->cellSizeHint(f);
    const QSize cellSymbolSize = d_legend->cellSymbolSizeHint();
    const QSize cellLabelSize = d_legend->cellLabelSizeHint(f);
    int bd = (cellSize.width() - cellSymbolSize.width()
                - cellLabelSize.width()) / 3;

    int centerAlignX = 0;
    int centerAlignY = 0;

    switch(d_layout->legendPos())
    {
        case Qwt::Top:
        case Qwt::Bottom:
            centerAlignX = ( rect.width() -
                d_legend->numCols() * cellSize.width() ) / 2;
            break;
        case Qwt::Left:
        case Qwt::Right:
            if ( d_legend->numRows() * cellSize.height() < pixRect.height() )
                centerAlignY = pixRect.y();
            break;
    }

    painter->save();
    painter->setFont(f);

    for ( int row = 0; row < d_legend->numRows(); row++ )
    {
        int y = rect.top() + centerAlignY + row * cellSize.height();

        for ( int col = 0; col < d_legend->numCols(); col++ )
        {
            int x = rect.left() + centerAlignX + col * cellSize.width();

            QRect symRect(x + bd, y, cellSymbolSize.width(),
                cellSize.height());

            QRect lblRect(x + bd + cellSymbolSize.width() + bd, y,
                cellLabelSize.width(), cellSize.height());

            QwtLegendItem *item =
                d_legend->d_item.at(d_legend->findIndex(row, col));
            if ( item )
            {
                QPen pen = item->pen();
                QPen cpPen = pen;

                QColor c = pfilter.color(pen.color(),
                    QwtPlotPrintFilter::Legend, item->key());
                cpPen.setColor(c);

                item->setPen(cpPen);
                item->draw(painter, symRect,
                    lblRect, d_legend->d_align);
                item->setPen(pen);
            }
        }
    }
    painter->restore();
#endif
}

/*!
  \brief Paint a scale into a given rectangle.
  Paint the scale into a given rectangle.

  \param painter Painter
  \param axis Axis
  \param borderDist Border distance
  \param baseDist Base distance
  \param rect Bounding rectangle
*/

void QwtPlot::printScale(QPainter *painter,
    int axis, int startDist, int endDist, int baseDist, const QRect &rect,
    const QwtPlotPrintFilter &pfilter) const
{
    if (!d_axisEnabled[axis])
        return;

    QwtScaleDraw::Orientation o;
    int x, y, w;

    switch(axis)
    {
        case yLeft:
        {
            x = rect.right() - baseDist + 1;
            y = rect.y() + startDist;
            w = rect.height() - startDist - endDist;
            o = QwtScaleDraw::Left;
            break;
        }
        case yRight:
        {
            x = rect.left() + baseDist;
            y = rect.y() + startDist;
            w = rect.height() - startDist - endDist;
            o = QwtScaleDraw::Right;
            break;
        }
        case xTop:
        {
            x = rect.left() + startDist;
            y = rect.bottom() - baseDist + 1;
            w = rect.width() - startDist - endDist;
            o = QwtScaleDraw::Top;
            break;
        }
        case xBottom:
        {
            x = rect.left() + startDist;
            y = rect.top() + baseDist;
            w = rect.width() - startDist - endDist;
            o = QwtScaleDraw::Bottom;
            break;
        }
        default:
            return;
    }

    const QwtScale *scale = d_scale[axis];

    painter->setPen(pfilter.color(Qt::black,
        QwtPlotPrintFilter::AxisTitle, axis));
    painter->setFont(pfilter.font(scale->titleFont(),
        QwtPlotPrintFilter::AxisTitle, axis));
    QwtScale::drawTitle(painter, o, rect,
        scale->titleAlignment(), scale->title());

    painter->setPen(pfilter.color(Qt::black,
        QwtPlotPrintFilter::AxisScale, axis));
    painter->setFont(pfilter.font(scale->font(),
        QwtPlotPrintFilter::AxisScale, axis));

    QwtScaleDraw *sd = (QwtScaleDraw *)scale->scaleDraw();
    int xSd = sd->x();
    int ySd = sd->y();
    int lengthSd = sd->length();

    sd->setGeometry(x, y, w, o);
    sd->draw(painter);
    sd->setGeometry(xSd, ySd, lengthSd, o); // reset previous values
}

/*!
  \brief Paint the pixrect into a given rectangle.
  Paint the pixrect into a given rectangle.

  \param painter Painter
  \param pixRect Bounding rectangle
  \param pfilter Print filter
  \sa QwtPlotPrintFilter
*/

void QwtPlot::printCanvas(QPainter *painter, const QwtDiMap map[],
        const QRect &pixRect, const QwtPlotPrintFilter &pfilter) const
{
    painter->setClipping(TRUE);
    QwtPainter::setClipRect(painter, QRect(pixRect.x(), pixRect.y(),
        pixRect.width() - 1, pixRect.height() - 1));

    if ( pfilter.options() & QwtPlotPrintFilter::PrintGrid )
    {
        QwtGrid cpGrid = d_grid;

        QPen cpPen = cpGrid.majPen();
        cpPen.setColor(pfilter.color(cpGrid.majPen().color(),
            QwtPlotPrintFilter::MajorGrid));
        cpGrid.setMajPen(cpPen);

        cpPen = cpGrid.minPen();
        cpPen.setColor(pfilter.color(cpGrid.minPen().color(),
            QwtPlotPrintFilter::MinorGrid));
        cpGrid.setMinPen(cpPen);

        cpGrid.draw(painter, pixRect,
            map[d_grid.xAxis()], map[d_grid.yAxis()]);
    }

    //
    //  draw curves
    //

    QIntDictIterator<QwtPlotCurve> itc(*d_curves);
    for (QwtPlotCurve *curCurve = itc.toFirst();
        curCurve != 0; curCurve = ++itc )
    {
        QwtCurve cpCurve = *curCurve;
        QPen cpPen = cpCurve.pen();
        cpPen.setColor(pfilter.color(cpPen.color(),
            QwtPlotPrintFilter::Curve, itc.currentKey()));
        cpCurve.setPen(cpPen);

        QwtSymbol s = cpCurve.symbol();
        cpPen = s.pen();
        cpPen.setColor(pfilter.color(cpPen.color(),
            QwtPlotPrintFilter::CurveSymbol, itc.currentKey()));
        s.setPen(cpPen);
        cpCurve.setSymbol(s);

        painter->save();
        cpCurve.draw(painter, map[curCurve->xAxis()], map[curCurve->yAxis()]);
        painter->restore();
    }

    //
    // draw markers
    //

    QIntDictIterator<QwtPlotMarker> itm(*d_markers);
    for (QwtPlotMarker *curMarker = itm.toFirst();
        curMarker != 0; curMarker = ++itm )
    {
        QwtMarker cpMarker = *curMarker;

        QPen cpPen = cpMarker.linePen();
        cpPen.setColor(pfilter.color(cpPen.color(),
            QwtPlotPrintFilter::Marker, itm.currentKey()));

        cpMarker.setLinePen(cpPen);

        cpPen = cpMarker.labelPen();
        cpPen.setColor(pfilter.color(cpPen.color(),
            QwtPlotPrintFilter::Marker, itm.currentKey()));
        cpMarker.setLabelPen(cpPen);

        QwtSymbol cpSym = cpMarker.symbol();
        cpPen = cpSym.pen();
        cpPen.setColor(pfilter.color(cpPen.color(),
            QwtPlotPrintFilter::MarkerSymbol, itm.currentKey()));
        cpSym.setPen(cpPen);

        cpMarker.setSymbol(cpSym);

        cpMarker.setFont(pfilter.font(cpMarker.font(),
            QwtPlotPrintFilter::Marker, itm.currentKey()));

        painter->save();
        cpMarker.draw(painter,
                map[curMarker->xAxis()].transform(curMarker->xValue()),
                map[curMarker->yAxis()].transform(curMarker->yValue()),
                pixRect);
        painter->restore();
    }
}
